

## This file is meant for experimentation with PDSim features and may not
## work for you.  Caveat emptor!!
#
from __future__ import division

# If being run from the folder that contains the PDSim source tree, 
# remove the current location from the python path and use the 
# site-packages version of PDSim
import sys, os
from math import pi

# If the following line is uncommented, python will try to use a local version
# of PDSim.  This is handy for debugging purposes.  Generally you want this line 
# commented out
#sys.path.insert(0, os.path.abspath('..'))

from PDSim.flow.flow_models import IsentropicNozzleWrapper
from PDSim.flow.flow import FlowPath
from PDSim.scroll import scroll_geo
from PDSim.scroll.core import Scroll
from PDSim.scroll.plots import plotScrollSet

from PDSim.core.containers import ControlVolume
from PDSim.core.core import Tube
from PDSim.plot.plots import debug_plots
from PDSim.core.motor import Motor
from PDSim.misc.hdf5 import HDF5Writer
from PDSim.misc.solvers import newton,Broyden
from multiprocessing import Process, Pipe, freeze_support, cpu_count, allow_connection_pickling
from CoolProp import State
from CoolProp import CoolProp as CP

import numpy as np
from math import sqrt
from scipy.optimize import fsolve, brentq
from matplotlib import pyplot as plt
import time
import subprocess

check_valve = False

class struct(object):
    pass

class InjectionPort(object):
    def __init__(self,**kwargs):
        self.__dict__.update(kwargs)
    
class InjectionLine(object):
    def __init__(self,**kwargs):
        self.__dict__.update(kwargs)
        
def BuildCompressor(Tevap, Tcond = 43.3, Ref='Propane'):
    ScrollComp=Scroll()
    ScrollComp.Ref = Ref
    #This runs if the module code is run directly
    
    ScrollComp.set_scroll_geo(100e-6,3.0,0.003,0.004) #Set the scroll wrap geometry
    ScrollComp.set_disc_geo('2Arc',r2=0)
    ScrollComp.geo.delta_flank = 15e-6
    ScrollComp.geo.delta_radial = 15e-6
    ScrollComp.omega = 3600/60*2*pi
    ScrollComp.Tamb = 298.0
    ScrollComp.eta_motor = 0.9
    
    #Set the bearing dimensions, using L/D of 1.0
    ScrollComp.mech = struct()
    ScrollComp.mech.D_upper_bearing = 0.035
    ScrollComp.mech.L_upper_bearing = 0.035
    ScrollComp.mech.c_upper_bearing = 20e-6
    ScrollComp.mech.D_crank_bearing = 0.035
    ScrollComp.mech.L_crank_bearing = 0.035
    ScrollComp.mech.c_crank_bearing = 20e-6
    ScrollComp.mech.D_lower_bearing = 0.035
    ScrollComp.mech.L_lower_bearing = 0.035
    ScrollComp.mech.c_lower_bearing = 20e-6
    ScrollComp.mech.thrust_friction_coefficient = 0.028 #From Chen thesis
    ScrollComp.mech.orbiting_scroll_mass = 2.0
    ScrollComp.mech.L_ratio_bearings = 3
    ScrollComp.mech.thrust_ID = ScrollComp.mech.D_crank_bearing*3
    ScrollComp.mech.mu_oil = 0.008
    
    ScrollComp.h_shell = 0.010
    ScrollComp.A_shell = 0.05
    ScrollComp.HTC = 0.5
    
    ScrollComp.geo.delta_suction_offset = 0.0e-3
    ScrollComp.geo.phi_ie_offset = 0.0
    
    
    ScrollComp.motor = Motor()
    ScrollComp.motor.set_eta(0.9)
    ScrollComp.motor.suction_fraction = 1.0
    
    Te = Tevap + 273.15
    Tc =  Tcond + 273.15
    Tin = Te + 11.1
    DT_sc = 7
    pe = CP.Props('P','T',Te,'Q',1.0,Ref)
    pc = CP.Props('P','T',Tc,'Q',1.0,Ref)
    inletState = State.State(Ref,{'T':Tin,'P':pe},phase = 'Gas')

    T2s = ScrollComp.guess_outlet_temp(inletState,pc)
    outletState = State.State(Ref,{'T':T2s,'P':pc},phase = 'Gas') 
    
    mdot_guess = 0.95*inletState.rho*ScrollComp.Vdisp*ScrollComp.omega/(2*pi)
    ScrollComp.mdot_guess = mdot_guess
    ScrollComp.add_tube(Tube(key1='inlet.1',
                             key2='inlet.2',
                             L=0.3,
                             ID=0.02,
                             mdot=mdot_guess, 
                             State1=inletState.copy(),
                             fixed=1,
                             TubeFcn=ScrollComp.TubeCode))
    ScrollComp.add_tube(Tube(key1='outlet.1',
                             key2='outlet.2',
                             L=0.3,
                             ID=0.02,
                             mdot=mdot_guess, 
                             State2=outletState.copy(),
                             fixed=2,
                             TubeFcn=ScrollComp.TubeCode))
    
    ScrollComp.auto_add_CVs(inletState, outletState)
    
    ScrollComp.auto_add_leakage(flankFunc = ScrollComp.FlankLeakage, 
                                radialFunc = ScrollComp.RadialLeakage)
    
    FP = FlowPath(key1='inlet.2', 
                  key2='sa', 
                  MdotFcn=IsentropicNozzleWrapper(),
                  )
    FP.A = pi*0.02**2/4
    ScrollComp.add_flow(FP)
    
    ScrollComp.add_flow(FlowPath(key1='sa', 
                                 key2='s1',
                                 MdotFcn=ScrollComp.SA_S1,
                                 MdotFcn_kwargs = dict(X_d = 1.0)
                                 )
                        )
    ScrollComp.add_flow(FlowPath(key1 = 'sa',
                                 key2 = 's2',
                                 MdotFcn = ScrollComp.SA_S2,
                                 MdotFcn_kwargs = dict(X_d = 1.0)
                                 )
                        )
             
    FP = FlowPath(key1='outlet.1', 
                  key2='dd', 
                  MdotFcn=IsentropicNozzleWrapper(),
                  )
    FP.A = pi*0.009**2/4
    ScrollComp.add_flow(FP)
    
    FP = FlowPath(key1='outlet.1', 
                  key2='ddd', 
                  MdotFcn=IsentropicNozzleWrapper(),
                  )
    FP.A = pi*0.009**2/4
    ScrollComp.add_flow(FP)
    
    ScrollComp.add_flow(FlowPath(key1='d1',
                                 key2='dd',
                                 MdotFcn=ScrollComp.D_to_DD))
    ScrollComp.add_flow(FlowPath(key1='d2',
                                 key2='dd',
                                 MdotFcn=ScrollComp.D_to_DD))
    return ScrollComp
    
def AddInjection(ScrollComp, Injection, d_injection_port = None):
    
    for iLine,Line in enumerate(Injection):
        p = CP.Props('P', 'T', Line.Tsat, 'Q', 1.0, ScrollComp.Ref)
        T = Line.Tsat + 3
        injState1 = State.State(ScrollComp.Ref, dict(T=T, P=p),phase = 'Gas')
        V_tube = 0.3*pi*0.006**2/4.0
        
        # The time-variant control volume
        ScrollComp.add_CV(ControlVolume(key ='injCV.'+str(iLine+1),
                                        VdVFcn = ScrollComp.V_injection,
                                        VdVFcn_kwargs = dict(V_tube = V_tube),
                                        initialState = injState1
                                        )
                          )
        ScrollComp.add_tube(Tube(key1='injection_line.'+str(iLine+1)+'.1',
                                 key2='injection_line.'+str(iLine+1)+'.2',
                                 L=0.3,ID=0.006,
                                 mdot=0.05*ScrollComp.mdot_guess, 
                                 State1=ScrollComp.CVs['injCV.'+str(iLine+1)].State.copy(),
                                 fixed=1,
                                 TubeFcn=ScrollComp.TubeCode
                                 )
                            )
        ScrollComp.add_flow(FlowPath(key1 = 'injection_line.'+str(iLine+1)+'.2',
                                         key2 ='injCV.'+str(iLine+1),
                                         MdotFcn=ScrollComp.IsentropicNozzleFMSafe,
                                         MdotFcn_kwargs = dict(A = pi*0.006**2/4,
                                                               DP_floor = 0.2)
                                         )
                            )
        
        for iPort,Port in enumerate(Line.Ports):
            
            #Make a set so that you can be sure that you only keep unique partners
            partners = set()
            
            #Check the partners over a pretty fine grid over one rotation
            # partners is a set so it doesnt allow duplication
            for tt in np.linspace(0, 2*pi, 1000):
                partners.add(ScrollComp._get_injection_CVkey(Port.phi, tt, Port.inner_outer))
            
            #Loop over the partners
            for partner in partners:
                #Injection flow paths
                if d_injection_port is None:
                    d_injection_port = 0.0025
                ScrollComp.add_flow(FlowPath(key1= partner,
                                             key2 ='injCV.'+str(iLine+1),
                                             MdotFcn=ScrollComp.Injection_to_Comp,
                                             MdotFcn_kwargs = dict(phi = Port.phi,
                                                                   inner_outer = Port.inner_outer,
                                                                   check_valve = Port.check_valve,
                                                                   A = pi*d_injection_port**2/4.0)
                                            )
                                    )

    return ScrollComp
    
    
def UninjectedCompressorRuns(Te, Tc, OneCycle = False):
    """
    The compressor runs without injection
    
    Does not use a separate process because these runs are pretty fast
    """
    
    for _Te in Te:
        for _Tc in Tc:
            scroll = BuildCompressor(_Te, _Tc)
            IS = InjectedStub()
            IS.scroll = scroll
            IS.RunCompressor(OneCycle)
            scroll = IS.scroll
            del scroll.FlowStorage
            
            HW = HDF5Writer()
            fName = 'UninjectedCompressorRuns_Te='+str(_Te)+'C'+'_Tc='+str(_Tc)+'C'
            HW.write_to_file(scroll, fName+'.h5')
        
class InjectedStub(object):
    
##     def AddInjection(self):
##         self.scroll = AddInjection(self.scroll)
        
    def RunCompressor(self, OneCycle):
        from time import clock
        t1=clock()
        self.scroll.RK45_eps = 1e-8
        self.scroll.EulerN = 20000
        self.scroll.HeunN = 6000
        
        self.scroll.eps_cycle = 0.006
        try:
            self.scroll.connect_callbacks(step_callback=self.scroll.step_callback, 
                                          endcycle_callback=self.scroll.endcycle_callback,
                                          heat_transfer_callback=self.scroll.heat_transfer_callback,
                                          lumps_energy_balance_callback=self.scroll.lump_energy_balance_callback)
            self.scroll.precond_solve(key_inlet='inlet.1',key_outlet='outlet.2',
                         solver_method='RK45',
                         UseNR = False, #Use Newton-Raphson ND solver to determine the initial state
                         OneCycle = OneCycle,
                         plot_every_cycle= False
                         )
        except:
            #debug_plots(self.scroll)
            raise
    
        print 'time taken',clock()-t1
        self.CycleCalcs()
        
    def CycleCalcs(self):
        #Collect all the tubes that are injection lines
        InjTubeList = []
        for Tube in self.scroll.Tubes:
            if Tube.key1.startswith('injection'):
                InjTubeList.append(Tube.key1)
        #Sort it so it is in the right order for sure although it should be anyway
        InjTubeList = sorted(InjTubeList)
        
        #Inlet and outlet enthalpy for the compressor
        self.scroll.h1 = self.scroll.Tubes['inlet.1'].State1.h
        self.scroll.h2 = self.scroll.Tubes['outlet.2'].State2.h
        mdot_suction = self.scroll.Tubes['inlet.1'].mdot
        self.scroll.h3 = CP.Props('H','P',self.scroll.Tubes['outlet.2'].State2.p,'Q',0,self.scroll.Ref)
        
        if not InjTubeList:
            #h4 and h3 are the same
            self.scroll.h4 = self.scroll.h3
            mdot_cond = mdot_suction
        else:
            #There is injection!
            
            #An objective function to find the right flash tank pressure
            def OBJECTIVE(p, x, h):
                #x is the ratio of the vapor to the liquid flow rates
                h4 = CP.Props('H','P',p,'Q',0, self.scroll.Ref)
                h6 = CP.Props('H','P',p,'Q',1, self.scroll.Ref)
                return (1+x)*h-(h4+x*h6)
            
            if len(InjTubeList) == 1:
                #The ratio of the injection flow rate to the suction flow rate
                mdot_inj1 = self.scroll.Tubes['injection_line.1.1'].mdot
                psat_inj1 = self.scroll.Tubes['injection_line.1.1'].State1.p
                x = [mdot_inj1/self.scroll.mdot]
                print 'x is', x
                #This enthalpy is for the flash pressure calculation - the saturation enthalpy corresponding to the condensing pressure
                h = [CP.Props('H','P',self.scroll.Tubes['outlet.1'].State1.p,'Q',0,self.scroll.Ref)]
                p0 = [psat_inj1]
                mdot_cond = mdot_suction+mdot_inj1
                self.scroll.pflash = [newton(OBJECTIVE,psat_inj1,args=(x[0],h[0],))]
            elif len(InjTubeList) == 2:
                self.scroll.pflash = [0,0]
                x = [0, 0]
                h = [0, 0]
                p0 = [0, 0]
                mdot_inj2 = self.scroll.Tubes['injection_line.2.1'].mdot
                mdot_inj1 = self.scroll.Tubes['injection_line.1.1'].mdot
                pinj2 = self.scroll.Tubes['injection_line.2.1'].State1.p
                pinj1 = self.scroll.Tubes['injection_line.1.1'].State1.p
                x[1] = mdot_inj2/(mdot_inj1+mdot_suction)
                x[0] = mdot_inj1/mdot_suction
                h[1] = self.scroll.h3
                self.scroll.pflash[1] = newton(OBJECTIVE,pinj2,args=(x[1],h[1]))
                h[0] = CP.Props('H','P',self.scroll.pflash[1],'Q',0,self.scroll.Ref)
                self.scroll.pflash[0] = newton(OBJECTIVE,pinj1,args=(x[0],h[0]))
                mdot_cond = mdot_suction + mdot_inj1 + mdot_inj2
                p0 = [pinj1,pinj2]
                print 'x is', x
                
            #The enthalpy corresponding to the saturated liquid for the lowest pressure injection line
            #This is the enthalpy used for the capacity calculations
            self.scroll.h4 = CP.Props('H','P',self.scroll.pflash[0],'Q',0,self.scroll.Ref)
            
            self.scroll.Tsat_inj = [CP.Props('T','P',p,'Q',1,self.scroll.Ref) for p in p0]
            self.scroll.Tflash = [CP.Props('T','P',p,'Q',1,self.scroll.Ref) for p in self.scroll.pflash] 
            self.scroll.pinjection = [p0]
            
            print 'Tflash', self.scroll.Tflash
            print 'Tsat_inj', self.scroll.Tsat_inj
        
        self.scroll.Capacity_c = self.scroll.mdot*(self.scroll.h1-self.scroll.h4)
        self.scroll.Capacity_h = mdot_cond*(self.scroll.h2-self.scroll.h3)
        self.scroll.COP_c = self.scroll.Capacity_c/self.scroll.Wdot_electrical
        self.scroll.COP_h = self.scroll.Capacity_h/self.scroll.Wdot_electrical
            
        if not InjTubeList:
            return 
        elif len(InjTubeList) == 1:
            print float(self.scroll.Tflash[0]) - float(self.scroll.Tsat_inj[0])
            return float(self.scroll.Tflash[0]) - float(self.scroll.Tsat_inj[0])
        else:
            return [p1-p2 for p1,p2 in zip(self.scroll.Tflash,self.scroll.Tsat_inj)]
        
    def OBJECTIVE(self, x, DTsh = 3.0, OneCycle = False, Symmetric = True, Nlines = 2, check_valve = True, Te = -30+273.15, Tc = 43.3+273.15, d_injection_port = None):
        """
        x: array of saturation temps [K] in the same order as the lines in scroll.InjectionLinesData
        Te : Evap temp [K]
        DTsh : injection line superheat [K]
        OneCycle: Just do one cycle?
        Symmetric : Are the ports symmetric?
        Nlines : Number of injection lines - 1 or 2 for now
        check_valve : Do the injection lines use check valves?
        """
        if isinstance(x, (float,int)):
            x = [x]
            
        scroll = BuildCompressor(Te,Tc)
        
        ILs = []
        #If one injection line, or two and symmetric:
        if Nlines == 1 or Nlines == 2:
            IL = InjectionLine()
            IL.Tsat = x[0]
            phi_o = scroll.geo.phi_oe-pi-2*pi-0.01
            
            #This is the outermost (lowest-pressure) injection
            IL.Ports=[]
            if Symmetric:
                IL.Ports.append(InjectionPort(phi = phi_o,
                                              inner_outer = 'o',
                                              check_valve = check_valve
                                              )
                                )
            IL.Ports.append(InjectionPort(phi = phi_o+pi,
                                          inner_outer = 'i',
                                          check_valve = check_valve
                                          )
                            )
            ILs.append(IL)
            
            #Do the second line for symmetric two lines
            if Nlines == 2:
                IL = InjectionLine()
                IL.Tsat = x[1]
                phi_o = scroll.geo.phi_os+2*pi+0.01
                
                IL.Ports=[]
                IL.Ports.append(InjectionPort(phi = phi_o,
                                              inner_outer = 'o',
                                              check_valve = check_valve
                                              )
                                )
                if Symmetric:
                    IL.Ports.append(InjectionPort(phi = phi_o+pi,
                                                  inner_outer = 'i',
                                                  check_valve = check_valve
                                                  )
                                    )
                ILs.append(IL)
            
        AddInjection(scroll, ILs, d_injection_port)
        scroll.InjectionLinesData = ILs
        
        print 'inputs to OBJECTIVE are',x
        
        self.scroll = scroll
        self.RunCompressor(OneCycle)
        resids = self.CycleCalcs()
        print 'resids to OBJECTIVE are',x,resids
        return resids

from multiprocessing import Process

class RedirectText2File(object):
    """
    An text output redirector
    """
    def __init__(self, file_name):
        self.file_name = file_name
    def write(self, string):
        fp = open(self.file_name,'a')
        fp.write(string)
        fp.close()
    def flush(self):
        return None
    
class OneLineWrapper(Process):
    def __init__(self, Te, Tc, check_valve, Nlines = 1, Symmetric = True, Tinj = None, d_injection_port = None):
        Process.__init__(self)
        self.Te = Te 
        self.Tc = Tc
        self.Tinj = Tinj
        self.d_injection_port = d_injection_port
        self.check_valve = check_valve
        self.Nlines = Nlines
        self.Symmetric = Symmetric
#        self.stdout_pipe = stdout_pipe
        
    def run(self):
        check_valve = self.check_valve
        check_valve_string = '_with_check_valves' if check_valve else '_no_check_valves'
        Nlines_string = '_one_line' if self.Nlines == 1 else '_two_lines'
        symmetric_string = '_symmetric' if self.Symmetric else '_asymmetric'
        Tinj_string = '' if self.Tinj is None else '_Ti='+str(self.Tinj)+'C'
        dinj_string = '' if self.d_injection_port is None else '_d_inj='+str(self.d_injection_port)+'m'
        fName = 'InjectedCompressor'+Nlines_string+symmetric_string+check_valve_string+'_Te='+str(self.Te)+'C'+'_Tc='+str(self.Tc)+'C'+Tinj_string+dinj_string
        
        # Any stdout or stderr output will be redirected to a file that is written in append mode
        redir = RedirectText2File(fName+'.log')
        sys.stdout = redir
        sys.stderr = redir
        
        InjectedComp = InjectedStub()
        if self.Tinj is not None and self.d_injection_port is not None:
            kwargs = dict(check_valve = self.check_valve,
                          Nlines = self.Nlines,
                          Symmetric = self.Symmetric,
                          Te = self.Te,
                          Tc = self.Tc,
                          DTsh = 3,
                          OneCycle = False,
                          d_injection_port = self.d_injection_port)
            InjectedComp.OBJECTIVE(self.Tinj+273.15,**kwargs)
        elif self.Nlines == 1:
            kwargs = dict(check_valve = self.check_valve,
                          Nlines = self.Nlines,
                          Symmetric = self.Symmetric,
                          Te = self.Te,
                          Tc = self.Tc,
                          DTsh = 3,
                          OneCycle = False)
            newton(InjectedComp.OBJECTIVE, self.Te+273.15+10, dx = 5.0, kwargs = kwargs, ytol = 0.1)
        else:
            # Use information about the temps to start close to the solution (this correlation was obtained by fitting a few points that solved properly)
            x0 = 0.980748151*self.Te+286.2501956, 0.667262145*self.Te+302.8818349
            #x0 = (0.70*self.Te+0.30*43.3)+273.15,(0.40*self.Te+0.60*43.3)+273.15
            kwargs = dict(check_valve = self.check_valve,
                          Nlines = self.Nlines,
                          Symmetric = self.Symmetric,
                          Te = self.Te,
                          Tc = self.Tc,
                          DTsh = 3,
                          OneCycle = False)
            Broyden(InjectedComp.OBJECTIVE, x0, dx = 0.1, kwargs = kwargs, ytol = 0.1)
            
        scroll = InjectedComp.scroll
        del scroll.FlowStorage
        
        HW = HDF5Writer()
        
        HW.write_to_file(scroll, fName+'.h5')
    
def InjectedCompressorRuns(Te = [10], 
                           Tc = [43.3],
                           Tinj = None,
                           d_injection_port = None,
                           OneCycle = False, 
                           check_valve = False,
                           Nlines = 1,
                           Symmetric = True):
    """
    A compressor with one line with two ports attached to it - the ports are 
    symmetric and located as far towards the outside of the compressor as possible
    """
#    IS = InjectedStub()
#    print IS.OBJECTIVE([260,280], DTsh=3.0, OneCycle=False, Symmetric=Symmetric, Nlines=2, check_valve=False, Te=-30)
#    print IS.OBJECTIVE([265,280], DTsh=3.0, OneCycle=False, Symmetric=Symmetric, Nlines=2, check_valve=False, Te=-30)
#    print IS.OBJECTIVE([260,285], DTsh=3.0, OneCycle=False, Symmetric=Symmetric, Nlines=2, check_valve=False, Te=-30)
    
    _running = True
    Nactive = 0
    Nmax = 2
    runs = []
    pipes = [Pipe(duplex = False) for T in Te]
        
    if Tinj is None:
        Tinj = [None]
    
    if d_injection_port is None:
        d_injection_port = [None]
    

    from multiprocessing import Queue
    q = Queue()
    for Tevap in Te:
        for Tcond in Tc:
            for Ti in Tinj:
                for d in d_injection_port:
                    if Ti is not None and d is not None:
                        q.put((Tevap,Tcond,dict(Ti=Ti,d=d) ))
                    else:
                        q.put((Tevap,Tcond))
        
    time.sleep(0.5)
    while _running:
        if Nactive < Nmax and not q.empty():
            Ts = q.get()
            if len(Ts) == 2:
                Te,Tc = Ts
                Ti = None
                d = None
            else:
                Te,Tc,others = Ts
                d = others['d']
                Ti = others['Ti']
            
            #Start the run
            p = OneLineWrapper(Te,Tc,check_valve,Nlines,Symmetric,Tinj = Ti, d_injection_port = d)
            p.daemon = True
            p.start()
            
            Nactive += 1
            runs.append(p)
            print 'now',Nactive,'runs active'
        else:
            #See if any are finished
            for run in runs:
                if not run.is_alive():
                    Nactive -= 1
                    runs.remove(run)
                    print 'now',Nactive,'runs active'
            if Nactive == 0:
                _running = False
        time.sleep(1.0)
        
if __name__=='__main__':
    
##     UninjectedCompressorRuns(Te = [-20], Tc = [43.3], OneCycle = False)
    
    Te = [-30,-20,-10,0] #,2.5,5,7.5,10
##     UninjectedCompressorRuns(Te = Te, Tc = [43.3], OneCycle = False)
    InjectedCompressorRuns(Te = [-20], Tc = [43.3], OneCycle = False, check_valve = False, Tinj = [-5,-2.5,0,2.5,5,7.5,10], d_injection_port = [0.0015,0.002,0.0025,0.003]) #
    InjectedCompressorRuns(Te = Te, Tc = [43.3], OneCycle = False, check_valve = True)
    InjectedCompressorRuns(Te = Te, Tc = [43.3], OneCycle = False, check_valve = True, Nlines = 2, Symmetric=True)
    
    #Tc = [20,30,40,50]
##     Tc = [50, 60]
##     UninjectedCompressorRuns(Te = [10], Tc = Tc, OneCycle = False)
##     InjectedCompressorRuns(Te = [10], Tc = Tc, OneCycle = False, check_valve = False)
##     InjectedCompressorRuns(Te = [10], Tc = Tc, OneCycle = False, check_valve = True)
##     InjectedCompressorRuns(Te = [-10], Tc = [43.3], OneCycle = False, check_valve = True, Nlines = 2, Symmetric=True)
##     InjectedCompressorRuns(Te = [10], Tc = Tc, OneCycle = False, check_valve = True, Nlines = 2, Symmetric=False)
##     InjectedCompressorRuns(Te = [10], Tc = Tc, OneCycle = False, check_valve = False, Nlines = 2, Symmetric=False)
